﻿using System;
using System.Windows.Forms;


namespace FrequencyAnalysis1
{
    public partial class Form1 : Form
    {
        public Form1()
        {
            InitializeComponent();
        }

        private void AnalyseButton_Click(object sender, EventArgs e)
        {
            if (!Updating)
            {
                Updating = true;
                LetterFrequencies Letters = new LetterFrequencies();
                string CipherText = CipherTextBox.Text;
                CipherText = CipherText.Trim();
                CipherText = CipherText.ToUpper();
                for (int CharNum = 0; CharNum < CipherText.Length; CharNum++)
                {
                    char CipherTextChar = CipherText[CharNum];
                    Letters.Add(CipherTextChar);
                }
                Letters.Sort();

                LetterFrequencies StandardLetters = LetterFrequencies.StandardLetterFrequencies();
                StandardLetters.Sort();

                string FrequencyText = "";
                FrequencyText += "Letters\t\tStandard Letters\r\n";
                FrequencyText += "Letter\tFrequency\tLetter\tFrequency";

                for (int LetterNum = 0; LetterNum < 26; LetterNum++)
                {
                    FrequencyText += "\r\n";

                    LetterFrequency ALetter = Letters.Letters[LetterNum];
                    FrequencyText += ALetter.Letter + "\t" + ALetter.Frequency.ToString("#,##0");

                    FrequencyText += "\t";

                    LetterFrequency AStandardLetter = StandardLetters.Letters[LetterNum];
                    FrequencyText += AStandardLetter.Letter + "\t" + AStandardLetter.Frequency.ToString("#,##0");
                }
                FrequenciesBox.Text = FrequencyText;
                FrequenciesBox.SelectAll();
                FrequenciesBox.SelectionTabs = new int[] { 64, 160, 224 };

                string KeyString = "";
                for (int KeyCharNum = 0; KeyCharNum < 26; KeyCharNum++)
                {
                    char ALetter = (char)(KeyCharNum + (int)'A');
                    int LetterNum = StandardLetters.Find(ALetter);
                    KeyString += Letters.Letters[LetterNum].Letter;
                }
                EncryptKeyBox.Text = KeyString;
                CleanKey(true);
                Updating = false;
            }
        }

        private void Form1_Load(object sender, EventArgs e)
        {
            if (!Updating)
            {
                Updating = true;
                EncryptKeyBox.Text = "ABCDEFGHIHJKLMNOPQRSTUVWXYZ";
                CleanKey(true);
                CipherTextBox.Text = "DVOO WLMV LM WVXRKSVIRMT GSRH SRWWVM NVHHZTV! DSB MLG ULIN Z GVZN DRGS BLFI UIRVMWH ZMW GZPV KZIG RM GSRH BVZI'H XIBKGLTIZKSB XLNKVGRGRLM?";
                Updating = false;
            }
        }

        private void ResetButton_Click(object sender, EventArgs e)
        {
            if (!Updating)
            {
                Updating = true;
                EncryptKeyBox.Text = "ABCDEFGHIHJKLMNOPQRSTUVWXYZ";
                CleanKey(true);
                CipherTextBox.Text = "DVOO WLMV LM WVXRKSVIRMT GSRH SRWWVM NVHHZTV! DSB MLG ULIN Z GVZN DRGS BLFI UIRVMWH ZMW GZPV KZIG RM GSRH BVZI'H XIBKGLTIZKSB XLNKVGRGRLM?";
                Updating = false;
            }
        }

        private void EncryptKeyBox_TextChanged(object sender, EventArgs e)
        {
            if (!Updating)
            {
                Updating = true;
                CleanKey(true);
                Updating = false;
            }
        }

        private void DecryptKeyBox_TextChanged(object sender, EventArgs e)
        {
            if (!Updating)
            {
                Updating = true;
                CleanKey(false);
                Updating = false;
            }
        }

        private void PlainButton_Click(object sender, EventArgs e)
        {
            if (!Updating)
            {
                Updating = true;
                EncryptKeyBox.Text = "ABCDEFGHIHJKLMNOPQRSTUVWXYZ";
                CleanKey(true);
                Updating = false;
            }
        }

        private void AtbashButton_Click(object sender, EventArgs e)
        {
            if (!Updating)
            {
                Updating = true;
                EncryptKeyBox.Text = "ZYXWVUTSRQPONMLKJIHGFEDCBA";
                CleanKey(true);
                Updating = false;
            }
        }

        private void RandomButton_Click(object sender, EventArgs e)
        {
            if (!Updating)
            {
                Updating = true;
                bool[] KeyCharsUsed = new bool[26];
                for (int CharNum = 0; CharNum < 26; CharNum++)
                {
                    KeyCharsUsed[CharNum] = false;
                }

                Random ARandom = new Random();
                string KeyString = "";
                for (int CharNum = 0; CharNum < 26; CharNum++)
                {
                    bool Found = false;
                    while (!Found)
                    {
                        int CharIndex = ARandom.Next(0, 26);
                        if (!(KeyCharsUsed[CharIndex]))
                        {
                            KeyString += (char)(CharIndex + (int)'A');
                            KeyCharsUsed[CharIndex] = true;
                            Found = true;
                        }
                    }
                    KeyCharsUsed[CharNum] = false;
                }

                EncryptKeyBox.Text = KeyString;
                CleanKey(true);
                Updating = false;
            }
        }

        private void SwapButton_Click(object sender, EventArgs e)
        {
            if (!Updating)
            {
                Updating = true;

                bool IsOK = true;

                string Swap1 = SwapBox1.Text.Trim().ToLower();
                if (IsOK)
                {
                    if (Swap1.Length != 1)
                    {
                        IsOK = false;
                    }
                }

                char SwapChar1 = ' ';
                if (IsOK)
                {
                    SwapChar1 = Swap1[0];
                    if (!(char.IsLetter(SwapChar1)))
                    {
                        IsOK = false;
                    }
                }

                string Swap2 = SwapBox2.Text.Trim().ToLower();
                if (IsOK)
                {
                    if (Swap2.Length != 1)
                    {
                        IsOK = false;
                    }
                }

                char SwapChar2 = ' ';
                if (IsOK)
                {
                    SwapChar2 = Swap2[0];
                    if (!(char.IsLetter(SwapChar2)))
                    {
                        IsOK = false;
                    }
                }

                if (IsOK)
                {
                    string KeyString = DecryptKeyBox.Text.Trim().ToLower();
                    string SwapString = "";
                    for (int CharNum = 0; CharNum < KeyString.Length; CharNum++)
                    {
                        char AChar = KeyString[CharNum];
                        if (AChar == SwapChar1)
                        {
                            AChar = SwapChar2;
                        }
                        else if (AChar == SwapChar2)
                        {
                            AChar = SwapChar1;
                        }

                        SwapString += AChar;
                    }

                    DecryptKeyBox.Text = SwapString;
                    CleanKey(false);
                }

                Updating = false;
            }
        }

        private void PlainTextBox_TextChanged(object sender, EventArgs e)
        {
            if (!Updating)
            {
                Updating = true;
                CipherTextBox.Text = "";
                Updating = false;
            }
        }

        private void CipherTextBox_TextChanged(object sender, EventArgs e)
        {
            if (!Updating)
            {
                Updating = true;
                PlainTextBox.Text = "";
                Updating = false;
            }
        }

        private void BlockButton_Click(object sender, EventArgs e)
        {
            if (!Updating)
            {
                Updating = true;
                string CipherText = CipherTextBox.Text;
                CipherText = CipherText.Trim();
                CipherText = CipherText.ToUpper();

                string BlockText = "";
                int Chars = 0;
                for (int CharNum = 0; CharNum < CipherText.Length; CharNum++)
                {
                    char CipherTextChar = CipherText[CharNum];
                    if (char.IsLetter(CipherTextChar))
                    {
                        if ((Chars != 0) && ((Chars % 5) == 0))
                        {
                            BlockText += " ";
                        }

                        BlockText += CipherTextChar;
                        Chars++;
                    }
                }

                Random ARandom = new Random();
                while ((Chars % 5) != 0)
                {
                    BlockText += (char)(ARandom.Next(0, 26) + (int)'A');
                    Chars++;
                }

                CipherTextBox.Text = BlockText;
                Updating = false;
            }
        }

        private void EncryptButton_Click(object sender, EventArgs e)
        {
            if (!Updating)
            {
                Updating = true;

                CleanKey(true);

                int Key = (int)(EncryptKeyBox.Text[0]);
                Key -= (int)'A';

                string PlainText = PlainTextBox.Text;
                PlainText = PlainText.Trim();
                PlainText = PlainText.ToLower();

                string CipherText = "";
                for (int CharNum = 0; CharNum < PlainText.Length; CharNum++)
                {
                    char PlainTextChar = PlainText[CharNum];
                    if (char.IsLetter(PlainTextChar))
                    {
                        int CharIndex = (int)PlainTextChar;
                        CharIndex -= (int)'a';
                        CharIndex = EncryptKeyCharIndices[CharIndex];
                        CharIndex += (int)'A';
                        char CipherTextChar = (char)CharIndex;
                        CipherText += CipherTextChar;
                    }
                    else
                    {
                        CipherText += PlainTextChar;
                    }
                }

                PlainTextBox.Text = PlainText;
                CipherTextBox.Text = CipherText;

                Updating = false;
            }
        }

        private void DecryptButton_Click(object sender, EventArgs e)
        {
            if (!Updating)
            {
                Updating = true;

                CleanKey(false);

                int Key = (int)(EncryptKeyBox.Text[0]);
                Key -= (int)'A';

                string CipherText = CipherTextBox.Text;
                CipherText = CipherText.Trim();
                CipherText = CipherText.ToUpper();

                string PlainText = "";
                for (int CharNum = 0; CharNum < CipherText.Length; CharNum++)
                {
                    char CipherTextChar = CipherText[CharNum];
                    if (char.IsLetter(CipherTextChar))
                    {
                        int CharIndex = (int)CipherTextChar;
                        CharIndex -= (int)'A';
                        CharIndex = DecryptKeyCharIndices[CharIndex];
                        CharIndex += (int)'a';
                        char PlainTextChar = (char)CharIndex;
                        PlainText += PlainTextChar;
                    }
                    else
                    {
                        PlainText += CipherTextChar;
                    }
                }

                PlainTextBox.Text = PlainText;
                CipherTextBox.Text = CipherText;

                Updating = false;
            }
        }

        private void CleanKey(bool FromEncryptKey)
        {
            string KeyString = (FromEncryptKey ? EncryptKeyBox.Text : DecryptKeyBox.Text);
            KeyString = KeyString.Trim();
            KeyString = KeyString.ToUpper();
            
            bool[] KeyCharsUsed = new bool[26];
            for (int KeyCharNum = 0; KeyCharNum < 26; KeyCharNum++)
            {
                KeyCharsUsed[KeyCharNum] = false;
            }

            int KeyCharIndex = 0;
            string CleanEncryptKeyString = "";
            string CleanDecryptKeyString = "";
            for (int CharNum = 0; CharNum < KeyString.Length; CharNum++)
            {
                char KeyChar = KeyString[CharNum];
                if (char.IsLetter(KeyChar))
                {
                    int CharIndex = (int)KeyChar;
                    CharIndex -= (int)'A';
                    if (!(KeyCharsUsed[CharIndex]))
                    {
                        if (FromEncryptKey)
                        {
                            CleanEncryptKeyString += (char)(CharIndex + (int)'A');
                            EncryptKeyCharIndices[KeyCharIndex] = CharIndex;
                            DecryptKeyCharIndices[CharIndex] = KeyCharIndex;
                        }
                        else
                        {
                            CleanDecryptKeyString += (char)(CharIndex + (int)'a');
                            EncryptKeyCharIndices[CharIndex] = KeyCharIndex;
                            DecryptKeyCharIndices[KeyCharIndex] = CharIndex;
                        }

                        KeyCharsUsed[CharIndex] = true;
                        KeyCharIndex++;
                    }
                }
            }

            for (int CharIndex = 0; CharIndex < 26; CharIndex++)
            {
                if (!(KeyCharsUsed[CharIndex]))
                {
                    if (FromEncryptKey)
                    {
                        CleanEncryptKeyString += (char)(CharIndex + (int)'A');
                        EncryptKeyCharIndices[KeyCharIndex] = CharIndex;
                        DecryptKeyCharIndices[CharIndex] = KeyCharIndex;
                    }
                    else
                    {
                        CleanDecryptKeyString += (char)(CharIndex + (int)'a');
                        EncryptKeyCharIndices[CharIndex] = KeyCharIndex;
                        DecryptKeyCharIndices[KeyCharIndex] = CharIndex;
                    }

                    KeyCharsUsed[CharIndex] = true;
                    KeyCharIndex++;
                }
            }

            for (int CharIndex = 0; CharIndex < 26; CharIndex++)
            {
                if (FromEncryptKey)
                {
                    CleanDecryptKeyString += (char)(DecryptKeyCharIndices[CharIndex] + (int)'a');
                }
                else
                {
                    CleanEncryptKeyString += (char)(EncryptKeyCharIndices[CharIndex] + (int)'A');
                }
            }

            EncryptKeyBox.Text = CleanEncryptKeyString;
            DecryptKeyBox.Text = CleanDecryptKeyString;
        }

        int[] EncryptKeyCharIndices = new int[26];
        int[] DecryptKeyCharIndices = new int[26];

        private bool Updating = false;
    }
}
